<?php

namespace NoccyLabs\Daemon\Module;

class Timer extends AbstractModule
{
    
    /** @var array Active timeouts */
    protected $activeTimeouts = [];

    /** @var array Active intervals */
    protected $activeIntervals = [];
    
    /**
     * Called to initialize the module.
     * 
     */
    public function onLoad()
    {
        $this->daemon->registerFunction("addInterval", [ $this, "addInterval" ]);
        $this->daemon->registerFunction("addTimeout", [ $this, "addTimeout" ]);
    }
    
    public function onUnload()
    {
    }
    
    /**
     * Called for each kernel tick (not CPU tick)
     * 
     */
    public function onTick()
    {
        $this->checkTimers();
    }
    
    public function getName()
    {
        return "Timer";
    }

    /**
     * Check and update timeouts and intervals
     *
     */
    protected function checkTimers()
    {
        foreach ($this->activeTimeouts as $index=>$timeout) {
            if (time() >= $timeout['next']) {
                call_user_func($timeout['callback'], $timeout['userdata']);
                unset($this->activeTimeouts[$index]);
            }
        }

        foreach ($this->activeIntervals as $index=>$interval) {
            if (time() >= $interval['next']) {
                call_user_func($interval['callback'], $interval['userdata']);
                $this->activeIntervals[$index]['next'] = time() + $interval['seconds'];
            }
        }
    }

    /**
     * Add a timeout to be called after the specified number of seconds. The timeout will
     * be removed once fired.
     *
     * @param int $seconds The number of seconds to wait before calling the callback
     * @param callable $callback Callback function
     * @param mixed|null $userdata Data to pass to the callback function
     */
    public function addTimeout($seconds, callable $callback, $userdata=null)
    {
        $this->activeTimeouts[] = [
            "seconds" => $seconds,
            "next" => time() + $seconds,
            "callback" => $callback,
            "userdata" => $userdata,
        ];
    }

    /**
     * Register a function to be called every specified number of seconds.
     *
     * @param int $seconds The number of seconds to wait between calling the callback
     * @param callable $callback Callback function
     * @param mixed|null $userdata Data to pass to the callback function
     */
    public function addInterval($seconds, callable $callback, $userdata=null)
    {
        $this->activeIntervals[] = [
            "seconds" => $seconds,
            "next" => time() + $seconds,
            "callback" => $callback,
            "userdata" => $userdata,
        ];
    }
    
}
